library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.all;

entity latch_tb is
end latch_tb;

architecture behavior of latch_tb is
  signal data_in_s: 	std_logic;
  signal enable_s:	std_logic;
  signal data_out_s:	std_logic;

begin
  
  latch : entity work.latch(behavioral)
    generic map (
      tpd => 100 ps )
    port map ( din => data_in_s,
               ena => enable_s,
               dout => data_out_s );
  
  testbench : process

    variable err_cnt_v: integer := 0;

  begin
    
    data_in_s <= '1';	  
    enable_s <= '0';
    wait for 20 ns;		   
    
    -- case 1	
    enable_s <= '1';
    wait for 5 ns;
    assert(data_out_s='1') report "Error1!" severity error;
    if (data_out_s/='1') then
      err_cnt_v := err_cnt_v + 1;
    end if;

    -- case 2
    data_in_s <= '0'; 
    wait for 20 ns;		
    assert(data_out_s='0') report "Error2!" severity error;
    if (data_out_s/='0') then
      err_cnt_v := err_cnt_v + 1;
    end if;
    
    -- case 3
    data_in_s <= '1'; 
    wait for 20 ns;		
    assert(data_out_s='1') report "Error3!" severity error;
    if (data_out_s/='1') then
      err_cnt_v := err_cnt_v + 1;
    end if;
    
    -- case 4
    enable_s <= '0';
    data_in_s <= '0'; 
    wait for 20 ns;						  
    assert(data_out_s='1') report "Error4!" severity error;
    if (data_out_s/='1') then
      err_cnt_v := err_cnt_v + 1;
    end if;

    -- case 5
    enable_s <= '1';
    wait for 5 ns;
    assert(data_out_s='0') report "Error5!" severity error;
    if (data_out_s/='0') then
      err_cnt_v := err_cnt_v + 1;
    end if;
    
    -- summary of all the tests
    if (err_cnt_v=0) then 			
      assert false 
        report "Testbench completed successfully!" 
        severity note; 
    else 
      assert true 
        report "Something wrong, try again" 
        severity error; 
    end if; 

    wait;
    
  end process testbench;
  
end behavior;
